@@ -1,6 +1,6 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v0.1.1
+## Version v0.1.2
 ## Copyright(c) 2020 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <@sitael.tokyo.deguest.jp>
 ## Created 2020/05/17
@@ -17,7 +17,7 @@ BEGIN
     use Filter::Util::Call;
     use Scalar::Util;
     # use Devel::Confess;
-    our $VERSION = 'v0.1.1';
+    our $VERSION = 'v0.1.2';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -34,11 +34,17 @@ sub import
     $hash->{debug_code} = 0 if( !CORE::exists( $hash->{debug_code} ) );
     filter_add( bless( $hash => ( ref( $this ) || $this ) ) );
 }
- 
+
+sub unimport
+{       
+    filter_del();
+}
+
 sub filter
 {
     my( $self ) = @_ ;
-    my( $status );
+    my( $status, $last_line );
+    my $line = 0;
     my $code = '';
     if( $self->{no_filter} )
     {
@@ -50,19 +56,59 @@ sub filter
     while( $status = filter_read() )
     {
         return( $status ) if( $status < 0 );
-        last if( /^__(DATA|END)__/ );
+        $line++;
+        if( /^__(?:DATA|END)__/ )
+        {
+            $last_line = $_;
+            last;
+        }
         $code .= $_;
         $_ = '';
     }
-    # $self->_message( 3, "Processing code:\n$code" );
-    my $doc = PPI::Document->new( \$code, readonly => 1 ) || die( "Unable to parse:\n$code\n" );
-    if( $doc = $self->_parse( $doc ) )
+    return( $line ) if( !$line );
+    unless( $status < 0 )
     {
-        $_ = "$doc";
+        ## $self->_message( 5, "Processing at line $line code:\n$code" );
+        my $doc = PPI::Document->new( \$code, readonly => 1 ) || die( "Unable to parse:\n$code\n" );
+        if( $doc = $self->_parse( $doc ) )
+        {
+            $_ = $doc->serialize;
+            # $doc->save( "./dev/debug-parsed.pl" );
+            # $status = 1;
+        }
+        ## Rollback
+        else
+        {
+            # $self->_message( 5, "Nothing found, restoring code to '$code'" );
+            $_ = $code;
+#             $status = -1;
+#             filter_del();
+        }
+        if( CORE::length( $last_line ) )
+        {
+            $_ .= $last_line;
+        }
     }
-    filter_del();
-    $status = 1;
-    return( $status );
+    unless( $status <= 0 )
+    {
+        while( $status = filter_read() )
+        {
+            return( $status ) if( $status < 0 );
+            $line++;
+        }
+    }
+    # $self->_message( 3, "Returning status '$line' with \$_ set to '$_'." );
+    if( $self->{debug_file} )
+    {
+        if( my $fh = IO::File->new( ">$self->{debug_file}" ) )
+        {
+            $fh->binmode( ':utf8' );
+            $fh->print( $_ );
+            $fh->close;
+        }
+    }
+    # filter_del();
+    return( $line );
 }
 
 sub _browse
@@ -145,6 +191,7 @@ sub _parse
     });
     return( $self->_error( "Failed to find any try-catch clause: $@" ) ) if( !defined( $ref ) );
     $self->_messagef( 3, "Found %d match(es)", scalar( @$ref ) );
+    return if( !scalar( @$ref ) );
     foreach my $this ( @$ref )
     {
         $self->_browse( $this ) if( $self->{debug} );
@@ -333,7 +380,7 @@ sub _parse
             my $finally_block = $fin_def->{block}->content;
             $finally_block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
             $fin_block = <<EOT;
-local \$Nice::Try::FINALLY = Nice\::Try\::ScopeGuard->_new(sub __FINALLY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ __FINALLY__CLOSE_NL__}, \@_);
+CORE::local \$Nice::Try::FINALLY = Nice\::Try\::ScopeGuard->_new(sub __FINALLY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ __FINALLY__CLOSE_NL__}, \@_);
 EOT
             $fin_block =~ s/\n/ /gs unless( $self->{debug_code} );
             $fin_block =~ s/__BLOCK_PLACEHOLDER__/$finally_block/gs;
@@ -380,17 +427,17 @@ EOT
             $try_block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
             
             my $try_sub = <<EOT;
-local \$Nice::Try::TRY = sub
+CORE::local \$Nice::Try::TRY = CORE::sub
 {
-    do __TRY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ };__TRY__CLOSE_NL__
-    return( \$Nice::Try::SENTINEL );
+    CORE::do __TRY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ };__TRY__CLOSE_NL__
+    CORE::return( \$Nice::Try::SENTINEL );
 };
-local ( \$Nice::Try::EXCEPTION, \$Nice::Try::DIED, \@Nice::Try::RETVAL );
-__FINALLY_BLOCK__ local \$Nice::Try::HAS_CATCH = $has_catch_clause;
-local \$Nice::Try::WANTARRAY = wantarray;
+CORE::local ( \$Nice::Try::EXCEPTION, \$Nice::Try::DIED, \@Nice::Try::RETVAL );
+__FINALLY_BLOCK__ CORE::local \$Nice::Try::HAS_CATCH = $has_catch_clause;
+CORE::local \$Nice::Try::WANTARRAY = CORE::wantarray;
 {
-    local \$\@;
-    eval 
+    CORE::local \$\@;
+    CORE::eval 
     {
         if( \$Nice::Try::WANTARRAY ) 
         {
@@ -405,7 +452,7 @@ local \$Nice::Try::WANTARRAY = wantarray;
             &\$Nice::Try::TRY;
         }
     };
-    \$Nice::Try::DIED = length( \$\@ ) ? 1 : 0;
+    \$Nice::Try::DIED = CORE::length( \$\@ ) ? 1 : 0;
     \$\@ =~ s/\\v+\$//g;
     \$Nice::Try::EXCEPTION = \$\@;
 };
@@ -523,10 +570,10 @@ EOT
                 $block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
                 my $catch_section = '';
                 my $catch_code = <<EOT;
-            local \$Nice::Try::CATCH = sub
+            CORE::local \$Nice::Try::CATCH = CORE::sub
             {
-                do __CATCH_OPEN_NL__{ __BLOCK_PLACEHOLDER__ }; __CATCH__CLOSE_NL__
-                return \$Nice::Try::SENTINEL;
+                CORE::do __CATCH_OPEN_NL__{ __BLOCK_PLACEHOLDER__ }; __CATCH__CLOSE_NL__
+                CORE::return \$Nice::Try::SENTINEL;
             };
             
             if( \$Nice::Try::WANTARRAY ) 
@@ -549,9 +596,9 @@ EOT
                     {
                         my $ex_class = $cdef->{class};
                         $catch_section = <<EOT;
-        ${cond}( ref( \$Nice::Try::EXCEPTION ) eq '$ex_class' )
+        ${cond}( CORE::ref( \$Nice::Try::EXCEPTION ) eq '$ex_class' )
         {
-            local \$\@ = \$Nice::Try::EXCEPTION;
+            CORE::local \$\@ = \$Nice::Try::EXCEPTION;
             my $ex_var = \$Nice::Try::EXCEPTION;
 $catch_code
         }
@@ -572,7 +619,7 @@ EOT
                         $catch_section = <<EOT;
         ${cond}
         {
-            local \$\@ = \$Nice::Try::EXCEPTION;
+            CORE::local \$\@ = \$Nice::Try::EXCEPTION;
             my $ex_var = \$Nice::Try::EXCEPTION;
 $catch_code
         }
@@ -586,7 +633,7 @@ EOT
                     $catch_section = <<EOT;
         ${cond}
         {
-            local \$\@ = \$Nice::Try::EXCEPTION;
+            CORE::local \$\@ = \$Nice::Try::EXCEPTION;
 $catch_code
         }
 EOT
@@ -627,9 +674,9 @@ EOT
     }
     else
     {
-        if( defined( (caller(0))[3] ) && (caller(0))[3] eq '(eval)' )
+        if( CORE::defined( (CORE::caller(0))[3] ) && (CORE::caller(0))[3] eq '(eval)' )
         {
-            die( \$Nice::Try::EXCEPTION );
+            CORE::die( \$Nice::Try::EXCEPTION );
         }
         else
         {
@@ -648,19 +695,19 @@ EOT
         push( @$repl, "\};" );
 
         ## If there is a catch clause, we put the final block here, if any
-        if( $has_catch_clause && length( $fin_block ) )
+        if( $has_catch_clause && CORE::length( $fin_block ) )
         {
             push( @$repl, $fin_block );
         }
         
         my $last_return_block = <<EOT;
-if( defined( \$Nice::Try::WANTARRAY ) and 
+if( CORE::defined( \$Nice::Try::WANTARRAY ) and 
     (
-      !ref( \$Nice::Try::RETVAL[0] ) or 
+      !CORE::ref( \$Nice::Try::RETVAL[0] ) or 
       !\$Nice::Try::RETVAL[0]->isa( 'Nice::Try::SENTINEL' ) 
     ) ) 
 {
-    return( \$Nice::Try::WANTARRAY ? \@Nice::Try::RETVAL : \$Nice::Try::RETVAL[0] );
+    CORE::return( \$Nice::Try::WANTARRAY ? \@Nice::Try::RETVAL : \$Nice::Try::RETVAL[0] );
 }
 EOT
         $last_return_block =~ s/\n/ /gs unless( $self->{debug_code} );
@@ -784,7 +831,7 @@ When run, this would produce, as one would expect:
 
 =head1 VERSION
 
-    v0.1.1
+    v0.1.2
 
 =head1 DESCRIPTION
 
@@ -969,7 +1016,7 @@ So, L<Nice::Try> is quite unique and fill the missing features, but because it i
 
 =head1 FINALLY
 
-Like with other language such as Jaba or JavaScript, the C<finally> block will be executed even if the C<try> or C<catch> block contains a return statement.
+Like with other language such as Java or JavaScript, the C<finally> block will be executed even if the C<try> or C<catch> block contains a return statement.
 
 This is useful to do some clean-up. For example:
 
